@@ -4,6 +4,7 @@ from __future__ import unicode_literals
 import os
 import shutil
 import tempfile
+from distutils.spawn import find_executable
 from os import path
 
 import pytest
@@ -15,9 +16,12 @@ from .. import mock
 from .testcases import DockerClientTestCase
 from .testcases import get_links
 from .testcases import pull_busybox
+from .testcases import SWARM_SKIP_CONTAINERS_ALL
+from .testcases import SWARM_SKIP_CPU_SHARES
 from compose import __version__
 from compose.config.types import VolumeFromSpec
 from compose.config.types import VolumeSpec
+from compose.const import IS_WINDOWS_PLATFORM
 from compose.const import LABEL_CONFIG_HASH
 from compose.const import LABEL_CONTAINER_NUMBER
 from compose.const import LABEL_ONE_OFF
@@ -25,12 +29,21 @@ from compose.const import LABEL_PROJECT
 from compose.const import LABEL_SERVICE
 from compose.const import LABEL_VERSION
 from compose.container import Container
+from compose.errors import OperationFailedError
 from compose.project import OneOffFilter
 from compose.service import ConvergencePlan
 from compose.service import ConvergenceStrategy
 from compose.service import NetworkMode
+from compose.service import PidMode
 from compose.service import Service
+from compose.utils import parse_nanoseconds_int
+from tests.integration.testcases import is_cluster
+from tests.integration.testcases import no_cluster
+from tests.integration.testcases import v2_1_only
+from tests.integration.testcases import v2_2_only
+from tests.integration.testcases import v2_3_only
 from tests.integration.testcases import v2_only
+from tests.integration.testcases import v3_only
 
 
 def create_and_start_container(service, **override_options):
@@ -39,6 +52,7 @@ def create_and_start_container(service, **override_options):
 
 
 class ServiceTest(DockerClientTestCase):
+
     def test_containers(self):
         foo = self.create_service('foo')
         bar = self.create_service('bar')
@@ -93,6 +107,7 @@ class ServiceTest(DockerClientTestCase):
         service.start_container(container)
         self.assertEqual('foodriver', container.get('HostConfig.VolumeDriver'))
 
+    @pytest.mark.skipif(SWARM_SKIP_CPU_SHARES, reason='Swarm --cpu-shares bug')
     def test_create_container_with_cpu_shares(self):
         service = self.create_service('db', cpu_shares=73)
         container = service.create_container()
@@ -105,6 +120,31 @@ class ServiceTest(DockerClientTestCase):
         container.start()
         self.assertEqual(container.get('HostConfig.CpuQuota'), 40000)
 
+    @v2_2_only()
+    def test_create_container_with_cpu_count(self):
+        self.require_api_version('1.25')
+        service = self.create_service('db', cpu_count=2)
+        container = service.create_container()
+        service.start_container(container)
+        self.assertEqual(container.get('HostConfig.CpuCount'), 2)
+
+    @v2_2_only()
+    @pytest.mark.skipif(not IS_WINDOWS_PLATFORM, reason='cpu_percent is not supported for Linux')
+    def test_create_container_with_cpu_percent(self):
+        self.require_api_version('1.25')
+        service = self.create_service('db', cpu_percent=12)
+        container = service.create_container()
+        service.start_container(container)
+        self.assertEqual(container.get('HostConfig.CpuPercent'), 12)
+
+    @v2_2_only()
+    def test_create_container_with_cpus(self):
+        self.require_api_version('1.25')
+        service = self.create_service('db', cpus=1)
+        container = service.create_container()
+        service.start_container(container)
+        self.assertEqual(container.get('HostConfig.NanoCpus'), 1000000000)
+
     def test_create_container_with_shm_size(self):
         self.require_api_version('1.22')
         service = self.create_service('db', shm_size=67108864)
@@ -112,6 +152,30 @@ class ServiceTest(DockerClientTestCase):
         service.start_container(container)
         self.assertEqual(container.get('HostConfig.ShmSize'), 67108864)
 
+    def test_create_container_with_init_bool(self):
+        self.require_api_version('1.25')
+        service = self.create_service('db', init=True)
+        container = service.create_container()
+        service.start_container(container)
+        assert container.get('HostConfig.Init') is True
+
+    @pytest.mark.xfail(True, reason='Option has been removed in Engine 17.06.0')
+    def test_create_container_with_init_path(self):
+        self.require_api_version('1.25')
+        docker_init_path = find_executable('docker-init')
+        service = self.create_service('db', init=docker_init_path)
+        container = service.create_container()
+        service.start_container(container)
+        assert container.get('HostConfig.InitPath') == docker_init_path
+
+    @pytest.mark.xfail(True, reason='Some kernels/configs do not support pids_limit')
+    def test_create_container_with_pids_limit(self):
+        self.require_api_version('1.23')
+        service = self.create_service('db', pids_limit=10)
+        container = service.create_container()
+        service.start_container(container)
+        assert container.get('HostConfig.PidsLimit') == 10
+
     def test_create_container_with_extra_hosts_list(self):
         extra_hosts = ['somehost:162.242.195.82', 'otherhost:50.31.209.229']
         service = self.create_service('db', extra_hosts=extra_hosts)
@@ -140,6 +204,34 @@ class ServiceTest(DockerClientTestCase):
         service.start_container(container)
         assert container.get('HostConfig.ReadonlyRootfs') == read_only
 
+    def test_create_container_with_blkio_config(self):
+        blkio_config = {
+            'weight': 300,
+            'weight_device': [{'path': '/dev/sda', 'weight': 200}],
+            'device_read_bps': [{'path': '/dev/sda', 'rate': 1024 * 1024 * 100}],
+            'device_read_iops': [{'path': '/dev/sda', 'rate': 1000}],
+            'device_write_bps': [{'path': '/dev/sda', 'rate': 1024 * 1024}],
+            'device_write_iops': [{'path': '/dev/sda', 'rate': 800}]
+        }
+        service = self.create_service('web', blkio_config=blkio_config)
+        container = service.create_container()
+        assert container.get('HostConfig.BlkioWeight') == 300
+        assert container.get('HostConfig.BlkioWeightDevice') == [{
+            'Path': '/dev/sda', 'Weight': 200
+        }]
+        assert container.get('HostConfig.BlkioDeviceReadBps') == [{
+            'Path': '/dev/sda', 'Rate': 1024 * 1024 * 100
+        }]
+        assert container.get('HostConfig.BlkioDeviceWriteBps') == [{
+            'Path': '/dev/sda', 'Rate': 1024 * 1024
+        }]
+        assert container.get('HostConfig.BlkioDeviceReadIOps') == [{
+            'Path': '/dev/sda', 'Rate': 1000
+        }]
+        assert container.get('HostConfig.BlkioDeviceWriteIOps') == [{
+            'Path': '/dev/sda', 'Rate': 800
+        }]
+
     def test_create_container_with_security_opt(self):
         security_opt = ['label:disable']
         service = self.create_service('db', security_opt=security_opt)
@@ -147,6 +239,15 @@ class ServiceTest(DockerClientTestCase):
         service.start_container(container)
         self.assertEqual(set(container.get('HostConfig.SecurityOpt')), set(security_opt))
 
+    # @pytest.mark.xfail(True, reason='Not supported on most drivers')
+    @pytest.mark.skipif(True, reason='https://github.com/moby/moby/issues/34270')
+    def test_create_container_with_storage_opt(self):
+        storage_opt = {'size': '1G'}
+        service = self.create_service('db', storage_opt=storage_opt)
+        container = service.create_container()
+        service.start_container(container)
+        self.assertEqual(container.get('HostConfig.StorageOpt'), storage_opt)
+
     def test_create_container_with_mac_address(self):
         service = self.create_service('db', mac_address='02:42:ac:11:65:43')
         container = service.create_container()
@@ -170,6 +271,24 @@ class ServiceTest(DockerClientTestCase):
         self.assertTrue(path.basename(actual_host_path) == path.basename(host_path),
                         msg=("Last component differs: %s, %s" % (actual_host_path, host_path)))
 
+    def test_create_container_with_healthcheck_config(self):
+        one_second = parse_nanoseconds_int('1s')
+        healthcheck = {
+            'test': ['true'],
+            'interval': 2 * one_second,
+            'timeout': 5 * one_second,
+            'retries': 5,
+            'start_period': 2 * one_second
+        }
+        service = self.create_service('db', healthcheck=healthcheck)
+        container = service.create_container()
+        remote_healthcheck = container.get('Config.Healthcheck')
+        assert remote_healthcheck['Test'] == healthcheck['test']
+        assert remote_healthcheck['Interval'] == healthcheck['interval']
+        assert remote_healthcheck['Timeout'] == healthcheck['timeout']
+        assert remote_healthcheck['Retries'] == healthcheck['retries']
+        assert remote_healthcheck['StartPeriod'] == healthcheck['start_period']
+
     def test_recreate_preserves_volume_with_trailing_slash(self):
         """When the Compose file specifies a trailing slash in the container path, make
         sure we copy the volume over when recreating.
@@ -194,6 +313,7 @@ class ServiceTest(DockerClientTestCase):
             'busybox', 'true',
             volumes={container_path: {}},
             labels={'com.docker.compose.test_image': 'true'},
+            host_config={}
         )
         image = self.client.commit(tmp_container)['Id']
 
@@ -223,13 +343,16 @@ class ServiceTest(DockerClientTestCase):
             image='busybox:latest',
             command=["top"],
             labels={LABEL_PROJECT: 'composetest'},
+            host_config={},
+            environment=['affinity:container=={}'.format(volume_container_1.id)],
         )
         host_service = self.create_service(
             'host',
             volumes_from=[
                 VolumeFromSpec(volume_service, 'rw', 'service'),
                 VolumeFromSpec(volume_container_2, 'rw', 'container')
-            ]
+            ],
+            environment=['affinity:container=={}'.format(volume_container_1.id)],
         )
         host_container = host_service.create_container()
         host_service.start_container(host_container)
@@ -266,9 +389,15 @@ class ServiceTest(DockerClientTestCase):
         self.assertIn('FOO=2', new_container.get('Config.Env'))
         self.assertEqual(new_container.name, 'composetest_db_1')
         self.assertEqual(new_container.get_mount('/etc')['Source'], volume_path)
-        self.assertIn(
-            'affinity:container==%s' % old_container.id,
-            new_container.get('Config.Env'))
+        if not is_cluster(self.client):
+            assert (
+                'affinity:container==%s' % old_container.id in
+                new_container.get('Config.Env')
+            )
+        else:
+            # In Swarm, the env marker is consumed and the container should be deployed
+            # on the same node.
+            assert old_container.get('Node.Name') == new_container.get('Node.Name')
 
         self.assertEqual(len(self.client.containers(all=True)), num_containers_before)
         self.assertNotEqual(old_container.id, new_container.id)
@@ -295,8 +424,13 @@ class ServiceTest(DockerClientTestCase):
                 ConvergencePlan('recreate', [orig_container]))
 
             assert new_container.get_mount('/etc')['Source'] == volume_path
-            assert ('affinity:container==%s' % orig_container.id in
-                    new_container.get('Config.Env'))
+            if not is_cluster(self.client):
+                assert ('affinity:container==%s' % orig_container.id in
+                        new_container.get('Config.Env'))
+            else:
+                # In Swarm, the env marker is consumed and the container should be deployed
+                # on the same node.
+                assert orig_container.get('Node.Name') == new_container.get('Node.Name')
 
             orig_container = new_container
 
@@ -409,18 +543,21 @@ class ServiceTest(DockerClientTestCase):
         )
 
         containers = service.execute_convergence_plan(ConvergencePlan('create', []), start=False)
-        self.assertEqual(len(service.containers()), 0)
-        self.assertEqual(len(service.containers(stopped=True)), 1)
+        service_containers = service.containers(stopped=True)
+        assert len(service_containers) == 1
+        assert not service_containers[0].is_running
 
         containers = service.execute_convergence_plan(
             ConvergencePlan('recreate', containers),
             start=False)
-        self.assertEqual(len(service.containers()), 0)
-        self.assertEqual(len(service.containers(stopped=True)), 1)
+        service_containers = service.containers(stopped=True)
+        assert len(service_containers) == 1
+        assert not service_containers[0].is_running
 
         service.execute_convergence_plan(ConvergencePlan('start', containers), start=False)
-        self.assertEqual(len(service.containers()), 0)
-        self.assertEqual(len(service.containers(stopped=True)), 1)
+        service_containers = service.containers(stopped=True)
+        assert len(service_containers) == 1
+        assert not service_containers[0].is_running
 
     def test_start_container_passes_through_options(self):
         db = self.create_service('db')
@@ -432,6 +569,7 @@ class ServiceTest(DockerClientTestCase):
         create_and_start_container(db)
         self.assertEqual(db.containers()[0].environment['FOO'], 'BAR')
 
+    @no_cluster('No legacy links support in Swarm')
     def test_start_container_creates_links(self):
         db = self.create_service('db')
         web = self.create_service('web', links=[(db, None)])
@@ -448,6 +586,7 @@ class ServiceTest(DockerClientTestCase):
                 'db'])
         )
 
+    @no_cluster('No legacy links support in Swarm')
     def test_start_container_creates_links_with_names(self):
         db = self.create_service('db')
         web = self.create_service('web', links=[(db, 'custom_link_name')])
@@ -464,6 +603,7 @@ class ServiceTest(DockerClientTestCase):
                 'custom_link_name'])
         )
 
+    @no_cluster('No legacy links support in Swarm')
     def test_start_container_with_external_links(self):
         db = self.create_service('db')
         web = self.create_service('web', external_links=['composetest_db_1',
@@ -482,6 +622,7 @@ class ServiceTest(DockerClientTestCase):
                 'db_3']),
         )
 
+    @no_cluster('No legacy links support in Swarm')
     def test_start_normal_container_does_not_create_links_to_its_own_service(self):
         db = self.create_service('db')
 
@@ -491,6 +632,7 @@ class ServiceTest(DockerClientTestCase):
         c = create_and_start_container(db)
         self.assertEqual(set(get_links(c)), set([]))
 
+    @no_cluster('No legacy links support in Swarm')
     def test_start_one_off_container_creates_links_to_its_own_service(self):
         db = self.create_service('db')
 
@@ -517,7 +659,7 @@ class ServiceTest(DockerClientTestCase):
         container = create_and_start_container(service)
         container.wait()
         self.assertIn(b'success', container.logs())
-        self.assertEqual(len(self.client.images(name='composetest_test')), 1)
+        assert len(self.client.images(name='composetest_test')) >= 1
 
     def test_start_container_uses_tagged_image_if_it_exists(self):
         self.check_build('tests/fixtures/simple-dockerfile', tag='composetest_test')
@@ -544,7 +686,10 @@ class ServiceTest(DockerClientTestCase):
         with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
             f.write("FROM busybox\n")
 
-        self.create_service('web', build={'context': base_dir}).build()
+        service = self.create_service('web', build={'context': base_dir})
+        service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
+
         assert self.client.inspect_image('composetest_web')
 
     def test_build_non_ascii_filename(self):
@@ -557,7 +702,9 @@ class ServiceTest(DockerClientTestCase):
         with open(os.path.join(base_dir.encode('utf8'), b'foo\xE2bar'), 'w') as f:
             f.write("hello world\n")
 
-        self.create_service('web', build={'context': text_type(base_dir)}).build()
+        service = self.create_service('web', build={'context': text_type(base_dir)})
+        service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
         assert self.client.inspect_image('composetest_web')
 
     def test_build_with_image_name(self):
@@ -586,19 +733,107 @@ class ServiceTest(DockerClientTestCase):
         with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
             f.write("FROM busybox\n")
             f.write("ARG build_version\n")
+            f.write("RUN echo ${build_version}\n")
 
         service = self.create_service('buildwithargs',
                                       build={'context': text_type(base_dir),
                                              'args': {"build_version": "1"}})
         service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
+        assert service.image()
+        assert "build_version=1" in service.image()['ContainerConfig']['Cmd']
+
+    def test_build_with_build_args_override(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write("FROM busybox\n")
+            f.write("ARG build_version\n")
+            f.write("RUN echo ${build_version}\n")
+
+        service = self.create_service('buildwithargs',
+                                      build={'context': text_type(base_dir),
+                                             'args': {"build_version": "1"}})
+        service.build(build_args_override={'build_version': '2'})
+        self.addCleanup(self.client.remove_image, service.image_name)
+
+        assert service.image()
+        assert "build_version=2" in service.image()['ContainerConfig']['Cmd']
+
+    def test_build_with_build_labels(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write('FROM busybox\n')
+
+        service = self.create_service('buildlabels', build={
+            'context': text_type(base_dir),
+            'labels': {'com.docker.compose.test': 'true'}
+        })
+        service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
+
+        assert service.image()
+        assert service.image()['Config']['Labels']['com.docker.compose.test'] == 'true'
+
+    @no_cluster('Container networks not on Swarm')
+    def test_build_with_network(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write('FROM busybox\n')
+            f.write('RUN ping -c1 google.local\n')
+
+        net_container = self.client.create_container(
+            'busybox', 'top', host_config=self.client.create_host_config(
+                extra_hosts={'google.local': '127.0.0.1'}
+            ), name='composetest_build_network'
+        )
+
+        self.addCleanup(self.client.remove_container, net_container, force=True)
+        self.client.start(net_container)
+
+        service = self.create_service('buildwithnet', build={
+            'context': text_type(base_dir),
+            'network': 'container:{}'.format(net_container['Id'])
+        })
+
+        service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
+
         assert service.image()
 
-    def test_start_container_stays_unpriviliged(self):
+    @v2_3_only()
+    @no_cluster('Not supported on UCP 2.2.0-beta1')  # FIXME: remove once support is added
+    def test_build_with_target(self):
+        self.require_api_version('1.30')
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write('FROM busybox as one\n')
+            f.write('LABEL com.docker.compose.test=true\n')
+            f.write('LABEL com.docker.compose.test.target=one\n')
+            f.write('FROM busybox as two\n')
+            f.write('LABEL com.docker.compose.test.target=two\n')
+
+        service = self.create_service('buildtarget', build={
+            'context': text_type(base_dir),
+            'target': 'one'
+        })
+
+        service.build()
+        assert service.image()
+        assert service.image()['Config']['Labels']['com.docker.compose.test.target'] == 'one'
+
+    def test_start_container_stays_unprivileged(self):
         service = self.create_service('web')
         container = create_and_start_container(service).inspect()
         self.assertEqual(container['HostConfig']['Privileged'], False)
 
-    def test_start_container_becomes_priviliged(self):
+    def test_start_container_becomes_privileged(self):
         service = self.create_service('web', privileged=True)
         container = create_and_start_container(service).inspect()
         self.assertEqual(container['HostConfig']['Privileged'], True)
@@ -631,20 +866,27 @@ class ServiceTest(DockerClientTestCase):
             '0.0.0.0:9001:9000/udp',
         ])
         container = create_and_start_container(service).inspect()
-        self.assertEqual(container['NetworkSettings']['Ports'], {
-            '8000/tcp': [
-                {
-                    'HostIp': '127.0.0.1',
-                    'HostPort': '8001',
-                },
-            ],
-            '9000/udp': [
-                {
-                    'HostIp': '0.0.0.0',
-                    'HostPort': '9001',
-                },
-            ],
-        })
+        assert container['NetworkSettings']['Ports']['8000/tcp'] == [{
+            'HostIp': '127.0.0.1',
+            'HostPort': '8001',
+        }]
+        assert container['NetworkSettings']['Ports']['9000/udp'][0]['HostPort'] == '9001'
+        if not is_cluster(self.client):
+            assert container['NetworkSettings']['Ports']['9000/udp'][0]['HostIp'] == '0.0.0.0'
+        # self.assertEqual(container['NetworkSettings']['Ports'], {
+        #     '8000/tcp': [
+        #         {
+        #             'HostIp': '127.0.0.1',
+        #             'HostPort': '8001',
+        #         },
+        #     ],
+        #     '9000/udp': [
+        #         {
+        #             'HostIp': '0.0.0.0',
+        #             'HostPort': '9001',
+        #         },
+        #     ],
+        # })
 
     def test_create_with_image_id(self):
         # Get image id for the current busybox:latest
@@ -672,6 +914,10 @@ class ServiceTest(DockerClientTestCase):
         service.scale(0)
         self.assertEqual(len(service.containers()), 0)
 
+    @pytest.mark.skipif(
+        SWARM_SKIP_CONTAINERS_ALL,
+        reason='Swarm /containers/json bug'
+    )
     def test_scale_with_stopped_containers(self):
         """
         Given there are some stopped containers and scale is called with a
@@ -732,15 +978,15 @@ class ServiceTest(DockerClientTestCase):
                 message="testing",
                 response={},
                 explanation="Boom")):
-
             with mock.patch('sys.stderr', new_callable=StringIO) as mock_stderr:
-                service.scale(3)
+                with pytest.raises(OperationFailedError):
+                    service.scale(3)
 
-        self.assertEqual(len(service.containers()), 1)
-        self.assertTrue(service.containers()[0].is_running)
-        self.assertIn(
-            "ERROR: for composetest_web_2  Cannot create container for service web: Boom",
-            mock_stderr.getvalue()
+        assert len(service.containers()) == 1
+        assert service.containers()[0].is_running
+        assert (
+            "ERROR: for composetest_web_2  Cannot create container for service"
+            " web: Boom" in mock_stderr.getvalue()
         )
 
     def test_scale_with_unexpected_exception(self):
@@ -792,7 +1038,8 @@ class ServiceTest(DockerClientTestCase):
         service = self.create_service('app', container_name='custom-container')
         self.assertEqual(service.custom_container_name, 'custom-container')
 
-        service.scale(3)
+        with pytest.raises(OperationFailedError):
+            service.scale(3)
 
         captured_output = mock_log.warn.call_args[0][0]
 
@@ -833,15 +1080,27 @@ class ServiceTest(DockerClientTestCase):
         self.assertEqual(container.get('HostConfig.NetworkMode'), 'host')
 
     def test_pid_mode_none_defined(self):
-        service = self.create_service('web', pid=None)
+        service = self.create_service('web', pid_mode=None)
         container = create_and_start_container(service)
         self.assertEqual(container.get('HostConfig.PidMode'), '')
 
     def test_pid_mode_host(self):
-        service = self.create_service('web', pid='host')
+        service = self.create_service('web', pid_mode=PidMode('host'))
         container = create_and_start_container(service)
         self.assertEqual(container.get('HostConfig.PidMode'), 'host')
 
+    @v2_1_only()
+    def test_userns_mode_none_defined(self):
+        service = self.create_service('web', userns_mode=None)
+        container = create_and_start_container(service)
+        self.assertEqual(container.get('HostConfig.UsernsMode'), '')
+
+    @v2_1_only()
+    def test_userns_mode_host(self):
+        service = self.create_service('web', userns_mode='host')
+        container = create_and_start_container(service)
+        self.assertEqual(container.get('HostConfig.UsernsMode'), 'host')
+
     def test_dns_no_value(self):
         service = self.create_service('web')
         container = create_and_start_container(service)
@@ -852,11 +1111,42 @@ class ServiceTest(DockerClientTestCase):
         container = create_and_start_container(service)
         self.assertEqual(container.get('HostConfig.Dns'), ['8.8.8.8', '9.9.9.9'])
 
+    def test_mem_swappiness(self):
+        service = self.create_service('web', mem_swappiness=11)
+        container = create_and_start_container(service)
+        self.assertEqual(container.get('HostConfig.MemorySwappiness'), 11)
+
+    def test_mem_reservation(self):
+        service = self.create_service('web', mem_reservation='20m')
+        container = create_and_start_container(service)
+        assert container.get('HostConfig.MemoryReservation') == 20 * 1024 * 1024
+
     def test_restart_always_value(self):
         service = self.create_service('web', restart={'Name': 'always'})
         container = create_and_start_container(service)
         self.assertEqual(container.get('HostConfig.RestartPolicy.Name'), 'always')
 
+    def test_oom_score_adj_value(self):
+        service = self.create_service('web', oom_score_adj=500)
+        container = create_and_start_container(service)
+        self.assertEqual(container.get('HostConfig.OomScoreAdj'), 500)
+
+    def test_group_add_value(self):
+        service = self.create_service('web', group_add=["root", "1"])
+        container = create_and_start_container(service)
+
+        host_container_groupadd = container.get('HostConfig.GroupAdd')
+        assert "root" in host_container_groupadd
+        assert "1" in host_container_groupadd
+
+    def test_dns_opt_value(self):
+        service = self.create_service('web', dns_opt=["use-vc", "no-tld-query"])
+        container = create_and_start_container(service)
+
+        dns_opt = container.get('HostConfig.DnsOptions')
+        assert 'use-vc' in dns_opt
+        assert 'no-tld-query' in dns_opt
+
     def test_restart_on_failure_value(self):
         service = self.create_service('web', restart={
             'Name': 'on-failure',
@@ -915,6 +1205,22 @@ class ServiceTest(DockerClientTestCase):
         }.items():
             self.assertEqual(env[k], v)
 
+    @v3_only()
+    def test_build_with_cachefrom(self):
+        base_dir = tempfile.mkdtemp()
+        self.addCleanup(shutil.rmtree, base_dir)
+
+        with open(os.path.join(base_dir, 'Dockerfile'), 'w') as f:
+            f.write("FROM busybox\n")
+
+        service = self.create_service('cache_from',
+                                      build={'context': base_dir,
+                                             'cache_from': ['build1']})
+        service.build()
+        self.addCleanup(self.client.remove_image, service.image_name)
+
+        assert service.image()
+
     @mock.patch.dict(os.environ)
     def test_resolve_env(self):
         os.environ['FILE_DEF'] = 'E1'
@@ -943,7 +1249,7 @@ class ServiceTest(DockerClientTestCase):
         with mock.patch.object(self.client, '_version', '1.20'):
             service = self.create_service('web')
             service_config = service._get_container_host_config({})
-            self.assertEquals(service_config['NetworkMode'], 'default')
+            self.assertEqual(service_config['NetworkMode'], 'default')
 
     def test_labels(self):
         labels_dict = {
@@ -989,7 +1295,7 @@ class ServiceTest(DockerClientTestCase):
         one_off_container = service.create_container(one_off=True)
         self.assertNotEqual(one_off_container.name, 'my-web-container')
 
-    @pytest.mark.skipif(True, reason="Broken on 1.11.0rc1")
+    @pytest.mark.skipif(True, reason="Broken on 1.11.0 - 17.03.0")
     def test_log_drive_invalid(self):
         service = self.create_service('web', logging={'driver': 'xxx'})
         expected_error_msg = "logger: no log driver named 'xxx' is registered"
@@ -1047,6 +1353,7 @@ def converge(service, strategy=ConvergenceStrategy.changed):
 
 
 class ConfigHashTest(DockerClientTestCase):
+
     def test_no_config_hash_when_one_off(self):
         web = self.create_service('web')
         container = web.create_container(one_off=True)
