@@ -4,7 +4,9 @@ from __future__ import unicode_literals
 import itertools
 
 import pytest
+import requests
 import six
+from docker.errors import APIError
 from six.moves.queue import Queue
 
 from compose.cli.log_printer import build_log_generator
@@ -56,6 +58,26 @@ def test_wait_on_exit():
     assert expected == wait_on_exit(mock_container)
 
 
+def test_wait_on_exit_raises():
+    status_code = 500
+
+    def mock_wait():
+        resp = requests.Response()
+        resp.status_code = status_code
+        raise APIError('Bad server', resp)
+
+    mock_container = mock.Mock(
+        spec=Container,
+        name='cname',
+        wait=mock_wait
+    )
+
+    expected = 'Unexpected API error for {} (HTTP code {})\n'.format(
+        mock_container.name, status_code,
+    )
+    assert expected in wait_on_exit(mock_container)
+
+
 def test_build_no_log_generator(mock_container):
     mock_container.has_api_logs = False
     mock_container.log_driver = 'none'
@@ -165,11 +187,13 @@ class TestConsumeQueue(object):
         assert next(generator) == 'b'
 
     def test_item_is_stop_with_cascade_stop(self):
+        """Return the name of the container that caused the cascade_stop"""
         queue = Queue()
-        for item in QueueItem.stop(), QueueItem.new('a'), QueueItem.new('b'):
+        for item in QueueItem.stop('foobar-1'), QueueItem.new('a'), QueueItem.new('b'):
             queue.put(item)
 
-        assert list(consume_queue(queue, True)) == []
+        generator = consume_queue(queue, True)
+        assert next(generator) is 'foobar-1'
 
     def test_item_is_none_when_timeout_is_hit(self):
         queue = Queue()
