@@ -98,7 +98,7 @@ class ContainerTest(unittest.TestCase):
         self.assertEqual(container.name_without_project, "custom_name_of_container")
 
     def test_inspect_if_not_inspected(self):
-        mock_client = mock.create_autospec(docker.Client)
+        mock_client = mock.create_autospec(docker.APIClient)
         container = Container(mock_client, dict(Id="the_id"))
 
         container.inspect_if_not_inspected()
@@ -150,6 +150,34 @@ class ContainerTest(unittest.TestCase):
         container = Container(None, self.container_dict, has_been_inspected=True)
         assert container.short_id == self.container_id[:12]
 
+    def test_has_api_logs(self):
+        container_dict = {
+            'HostConfig': {
+                'LogConfig': {
+                    'Type': 'json-file'
+                }
+            }
+        }
+
+        container = Container(None, container_dict, has_been_inspected=True)
+        assert container.has_api_logs is True
+
+        container_dict['HostConfig']['LogConfig']['Type'] = 'none'
+        container = Container(None, container_dict, has_been_inspected=True)
+        assert container.has_api_logs is False
+
+        container_dict['HostConfig']['LogConfig']['Type'] = 'syslog'
+        container = Container(None, container_dict, has_been_inspected=True)
+        assert container.has_api_logs is False
+
+        container_dict['HostConfig']['LogConfig']['Type'] = 'journald'
+        container = Container(None, container_dict, has_been_inspected=True)
+        assert container.has_api_logs is True
+
+        container_dict['HostConfig']['LogConfig']['Type'] = 'foobar'
+        container = Container(None, container_dict, has_been_inspected=True)
+        assert container.has_api_logs is False
+
 
 class GetContainerNameTestCase(unittest.TestCase):
 
