@@ -10,6 +10,8 @@ from .. import mock
 from .. import unittest
 from compose.config.config import Config
 from compose.config.types import VolumeFromSpec
+from compose.const import COMPOSEFILE_V1 as V1
+from compose.const import COMPOSEFILE_V2_0 as V2_0
 from compose.const import LABEL_SERVICE
 from compose.container import Container
 from compose.project import Project
@@ -19,11 +21,11 @@ from compose.service import Service
 
 class ProjectTest(unittest.TestCase):
     def setUp(self):
-        self.mock_client = mock.create_autospec(docker.Client)
+        self.mock_client = mock.create_autospec(docker.APIClient)
 
-    def test_from_config(self):
+    def test_from_config_v1(self):
         config = Config(
-            version=None,
+            version=V1,
             services=[
                 {
                     'name': 'web',
@@ -36,6 +38,8 @@ class ProjectTest(unittest.TestCase):
             ],
             networks=None,
             volumes=None,
+            secrets=None,
+            configs=None,
         )
         project = Project.from_config(
             name='composetest',
@@ -51,7 +55,7 @@ class ProjectTest(unittest.TestCase):
 
     def test_from_config_v2(self):
         config = Config(
-            version=2,
+            version=V2_0,
             services=[
                 {
                     'name': 'web',
@@ -64,6 +68,8 @@ class ProjectTest(unittest.TestCase):
             ],
             networks=None,
             volumes=None,
+            secrets=None,
+            configs=None,
         )
         project = Project.from_config('composetest', config, None)
         self.assertEqual(len(project.services), 2)
@@ -162,7 +168,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[{
                     'name': 'test',
                     'image': 'busybox:latest',
@@ -170,6 +176,8 @@ class ProjectTest(unittest.TestCase):
                 }],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         assert project.get_service('test')._get_volumes_from() == [container_id + ":rw"]
@@ -188,7 +196,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[
                     {
                         'name': 'vol',
@@ -202,6 +210,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         assert project.get_service('test')._get_volumes_from() == [container_name + ":rw"]
@@ -213,7 +223,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=None,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[
                     {
                         'name': 'vol',
@@ -227,6 +237,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         with mock.patch.object(Service, 'containers') as mock_return:
@@ -351,7 +363,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V1,
                 services=[
                     {
                         'name': 'test',
@@ -360,6 +372,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         service = project.get_service('test')
@@ -374,7 +388,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[
                     {
                         'name': 'test',
@@ -384,6 +398,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         service = project.get_service('test')
@@ -403,7 +419,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[
                     {
                         'name': 'aaa',
@@ -417,6 +433,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
 
@@ -428,7 +446,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=2,
+                version=V2_0,
                 services=[
                     {
                         'name': 'foo',
@@ -437,6 +455,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
 
@@ -447,7 +467,7 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=2,
+                version=V2_0,
                 services=[
                     {
                         'name': 'foo',
@@ -457,6 +477,8 @@ class ProjectTest(unittest.TestCase):
                 ],
                 networks={'custom': {}},
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
 
@@ -480,13 +502,15 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version=None,
+                version=V2_0,
                 services=[{
                     'name': 'web',
                     'image': 'busybox:latest',
                 }],
                 networks=None,
                 volumes=None,
+                secrets=None,
+                configs=None,
             ),
         )
         self.assertEqual([c.id for c in project.containers()], ['1'])
@@ -496,13 +520,15 @@ class ProjectTest(unittest.TestCase):
             name='test',
             client=self.mock_client,
             config_data=Config(
-                version='2',
+                version=V2_0,
                 services=[{
                     'name': 'web',
                     'image': 'busybox:latest',
                 }],
                 networks={'default': {}},
                 volumes={'data': {}},
+                secrets=None,
+                configs=None,
             ),
         )
         self.mock_client.remove_network.side_effect = NotFound(None, None, 'oops')
